-- Raid Roster information
--
-- This builds two arrays, one for raid/party members
-- one for any mobs targeted by raid/party members
--
-- These are only generated once per update then every other
-- window can just refer to these as needed

-- The arrays
OZ_RaidRoster = {}
OZ_PlayerParty = nil


-- Functions in here:
-- OZ_SetupRaidRoster()		- Initialising the data
-- OZ_UpdateRoster()		- Fill in current raid members
-- OZ_UpdateTargets()		- Find all unique targets


OZ_ICON_LIST = {
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0,	 0,0.25,0.25},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.25,0,0.5, 0.25},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.5, 0,0.75,0.25},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.75,0,1.0, 0.25},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0,	 0.25,0.25,0.5},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.25,0.25,0.5, 0.5},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.5, 0.25,0.75,0.5},
					{"Interface\\TargetingFrame\\UI-RaidTargetingIcons",0.75,0.25,1.0, 0.5},
					{"Interface\\GroupFrame\\UI-Group-LeaderIcon",0,0,1,1},
					{"Interface\\GroupFrame\\UI-Group-MasterLooter",0,0,1,1},
				};

function OZ_SetupRaidRoster()
	local x

	OZ_RaidRoster = {
		["nMembers"]	= 0,
		["member"]		= {},
		["nTargets"]	= 0,
		["target"]		= {},
	}

	for x = 1 , 40 do
		OZ_RaidRoster.member[x] = {
			["name"]		= "name"..x,
			["unit"]		= "player",
			["rank"]		= 0,
			["subgroup"]	= 0,
			["level"]		= 0,
			["class"]		= "Mage",
			["fileName"]	= "MAGE",
			["zone"]		= "Offline",
			["online"]		= nil,
			["isDead"]		= nil,

			["unit"]		= "player",
			["colour"]		= {["r"]=1.0, ["g"]=0.0, ["b"]=0.0, ["a"]=1.0},

			["range"]		= nil,
			["health"]		= 0,
			["maxHealth"]	= 1000,
			["mana"]		= 0,
			["maxMana"]		= 1000,
		};
	end
	
	for x = 1 , 10 do
		OZ_RaidRoster.target[x] = {
			["name"]		= "name"..x,
			["unit"]		= "playertarget",
		};
	end
end

-- Fill in bar data for window 'n'
function OZ_GetMemberData(n, unit)
	if(unit == nil) then
		return
	end
	local class,fileName = UnitClass(unit)
	OZ_RaidRoster.member[n].unit 		= unit
	OZ_RaidRoster.member[n].name		= UnitName(unit)
	OZ_RaidRoster.member[n].rank		= 0
	OZ_RaidRoster.member[n].subgroup	= 1
	OZ_RaidRoster.member[n].level		= UnitLevel(unit)
	OZ_RaidRoster.member[n].class		= class
	OZ_RaidRoster.member[n].fileName	= fileName
	OZ_RaidRoster.member[n].zone		= nil
	OZ_RaidRoster.member[n].online		= UnitIsConnected(unit)
	OZ_RaidRoster.member[n].isDead		= UnitIsDeadOrGhost(unit)

	OZ_SetExtraMemberData(n)
end

function OZ_SetExtraMemberData(n)
	local m, c
	local unit = OZ_RaidRoster.member[n].unit
	OZ_RaidRoster.member[n].isDead		= UnitIsDeadOrGhost(unit)
	if( (OZ_RaidRoster.member[n].online) and (not OZ_RaidRoster.member[n].isDead))then
		m,c = UnitManaMax( unit ),UnitMana( unit )
		OZ_RaidRoster.member[n].mana	= c
		OZ_RaidRoster.member[n].maxMana	= m
		m,c = UnitHealthMax( unit ),UnitHealth( unit )
		OZ_RaidRoster.member[n].health		= c
		OZ_RaidRoster.member[n].maxHealth	= m
	else
		OZ_RaidRoster.member[n].maxHealth = 1
		OZ_RaidRoster.member[n].health	= 0
		OZ_RaidRoster.member[n].maxMana	= 1
		OZ_RaidRoster.member[n].health	= 0
	end
	OZ_RaidRoster.member[n].range	= CheckInteractDistance(unit, 4)

--	if(OZ_RangedActionSlot)then
--		local range2 = IsActionInRange(OZ_RangedActionSlot)
--		if(range2 == 1)then
--			-- Healer class, and member is in healing range
--			OZ_RaidRoster.member[n].range = 1
---		else
--			DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."action = "..OZ_RangedActionSlot);
--			if(range2)then
--				DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."return = "..range2);
--			end
--			-- out of range (or fubar action id)
--			if(OZ_RaidRoster.member[n].range)then
--				-- Eeep - we ARE in 30 yards, but range test failed - so fubar ID
--				-- Re-scan
--				OZ_FindHealRange()
--			end
--		end
--	end

	if( (OZ_RaidRoster.member[n].range) and (CheckInteractDistance(unit, 1)) )then
		OZ_RaidRoster.member[n].range = 2
	end
end

function OZ_UpdateRoster()
	local i
	OZ_PlayerParty = nil

--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."OzRaid: Updating roster");
	local nMembers = GetNumRaidMembers()
	if( nMembers == 0 ) then

		-- Not a raid - either lone or in party
		-- Add ourselves...
		OZ_PlayerParty = 1
		OZ_RaidRoster.nMembers = 1
		OZ_GetMemberData(1,"player")
		if(UnitIsPartyLeader("player")) then
			OZ_RaidRoster.member[1].rank = 2
		end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."OzRaid: PARTY: Add me");

		nMembers = GetNumPartyMembers()
		if(nMembers > 0) then
			for i = 1,nMembers do
--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."OzRaid: PARTY: Add party"..i);
				OZ_RaidRoster.nMembers = OZ_RaidRoster.nMembers + 1
				OZ_GetMemberData(OZ_RaidRoster.nMembers, "party"..i)
				if(UnitIsPartyLeader("party"..i)) then
					OZ_RaidRoster.member[OZ_RaidRoster.nMembers].rank = 2
				end
			end
		end
	else
		OZ_RaidRoster.nMembers = 0;
		for i = 1,40 do
			local name,rank,subgroup,level,class,fileName,zone,online,isDead
			name,rank,subgroup,level,class,fileName,zone,online,isDead = GetRaidRosterInfo(i)
			if(name) then
--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."OzRaid: RAID: Add "..name);
				OZ_RaidRoster.nMembers = OZ_RaidRoster.nMembers + 1
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].unit = "raid"..i

				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].name		= name
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].rank		= rank
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].subgroup	= subgroup
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].level		= level
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].class		= class
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].fileName	= fileName
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].zone		= zone
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].online		= online
				OZ_RaidRoster.member[OZ_RaidRoster.nMembers].isDead		= isDead

				if( UnitIsUnit("player", OZ_RaidRoster.member[OZ_RaidRoster.nMembers].unit) )then
					OZ_PlayerParty = subgroup
--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."Mygroup = "..subgroup);
				end

--				OZ_SetExtraMemberData(OZ_RaidRoster.nMembers)
			end
		end
	end
end

function OZ_UpdateTargets()
	local i
	OZ_RaidRoster.nTargets = 0;
	local m = OZ_RaidRoster.nMembers
	if( m>0 ) then
		for i = 1,m do
			OZ_UpdateTargetData( OZ_RaidRoster.member[i].unit )
		end
	end
end

function OZ_UpdateTargetData(unit)
	local t = unit.."target"
	if( UnitExists(t) and
		(UnitCanAttack("player",t)) and
		(not UnitIsCivilian(t)) and
		(not UnitIsDead(t)) ) then

		-- We have a valid target
		-- check if we have already counted this one...
		if( OZ_RaidRoster.nTargets > 0 ) then
			for n = 1,OZ_RaidRoster.nTargets do
				if( UnitIsUnit( t, OZ_RaidRoster.target[n].unit ) ) then
					return
				end
			end
		end

		-- Ok, this target is a mob, and is notyet in the list
		OZ_RaidRoster.nTargets = OZ_RaidRoster.nTargets + 1
		if(not OZ_RaidRoster.target[OZ_RaidRoster.nTargets])then
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets] = {}
		end
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].unit		= t
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].name		= UnitName(t)
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].level		= UnitLevel(t)
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].rank		= 0
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].subgroup	= 0
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].class		= "Mob"
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].fileName	= "TARGET"
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].zone		= nil
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].online		= 1
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].isDead		= UnitIsDead(t)

		local m, c
		if( not OZ_RaidRoster.target[OZ_RaidRoster.nTargets].isDead )then
			m,c = UnitManaMax( t ),UnitMana( t )
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].mana		= c
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].maxMana	= m

			m,c = UnitHealthMax( t ),UnitHealth( t )
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].health		= c
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].maxHealth	= m
		else
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].health		= 0
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].maxHealth	= 100
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].mana	= 0
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].maxMana	= 100
		end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033CCFF".."OzRaid: Health =  "..OZ_RaidRoster.target[OZ_RaidRoster.nTargets].health);
		OZ_RaidRoster.target[OZ_RaidRoster.nTargets].range	= CheckInteractDistance(t, 4)
		if( (OZ_RaidRoster.target[OZ_RaidRoster.nTargets].range) and (CheckInteractDistance(t, 1)) )then
			OZ_RaidRoster.target[OZ_RaidRoster.nTargets].range = 2
		end
	end
end
