---------------------------------------------------------------
-- OZ Raid sort function
-- The earlier stages have filled in 'OZ_Input' with the data
-- to display, and the sort routines set a weight for each entry.
--
-- This function then builds the [weight,index] data for the sort,
-- runs the sort, and then builds the bar data for this window
---------------------------------------------------------------
OZ_SortMap = {};

function OZ_InitSortTable(n)
	local i
	for i = 1,n do
		OZ_SortMap[i*2] = i
	end
end

function OZ_DoSort()
	local i
	for i = 1,OZ_Input.nBars do
		OZ_SortMap[i*2 - 1] = OZ_Input.bar[OZ_SortMap[i*2]].sortWeight
	end
--	OZ_SortMap = OZ_internalMergeSort(OZ_Input.nBars * 2, OZ_SortMap)
	OZ_internalMergeSort2(OZ_Input.nBars * 2, OZ_SortMap)
end

function OZ_FinaliseSort(n)
	-- Now copy the input data to the output 'OZ_Bars' array
	local i
	for i = 1,OZ_Input.nBars do
		OZ_Bars[n].bar[i].roster	= OZ_Input.bar[ OZ_SortMap[i*2] ].roster
		OZ_Bars[n].bar[i].target	= OZ_Input.bar[ OZ_SortMap[i*2] ].target			
		OZ_Bars[n].bar[i].current	= OZ_Input.bar[ OZ_SortMap[i*2] ].current			
		OZ_Bars[n].bar[i].max		= OZ_Input.bar[ OZ_SortMap[i*2] ].max		
		OZ_Bars[n].bar[i].value		= OZ_Input.bar[ OZ_SortMap[i*2] ].value
		OZ_Bars[n].bar[i].class		= OZ_Input.bar[ OZ_SortMap[i*2] ].class
		if(OZ_Bars[n].bar[i].roster > 0)then
			OZ_Bars[n].bar[i].unit = OZ_RaidRoster.member[OZ_Bars[n].bar[i].roster].unit
		elseif(OZ_Bars[n].bar[i].target > 0)then
			OZ_Bars[n].bar[i].unit = OZ_RaidRoster.target[OZ_Bars[n].bar[i].target].unit
		else
			OZ_Bars[n].bar[i].unit = nil
		end
		OZ_Bars[n].bar[i].buffs		= OZ_Input.bar[ OZ_SortMap[i*2] ].buffs
		OZ_Bars[n].bar[i].debuff	= OZ_Input.bar[ OZ_SortMap[i*2] ].debuff
	end
	OZ_Bars[n].nBars = OZ_Input.nBars
end

function OZ_MergeSort(n)
	-- First we build an array of indexes & values to sort
	local data = {};
	local i
	for i = 1,OZ_Input.nBars do
		data[i*2 - 1]	= OZ_Input.bar[i].sortWeight
		data[i*2]		= i
	end

	-- Do the sort
	data = OZ_internalMergeSort(OZ_Input.nBars * 2, data)

	-- Now copy the input data to the output 'OZ_Bars' array
	for i = 1,OZ_Input.nBars do
		OZ_Bars[n].bar[i].roster	= OZ_Input.bar[ data[i*2] ].roster
		OZ_Bars[n].bar[i].target	= OZ_Input.bar[ data[i*2] ].target			
		OZ_Bars[n].bar[i].current	= OZ_Input.bar[ data[i*2] ].current			
		OZ_Bars[n].bar[i].max		= OZ_Input.bar[ data[i*2] ].max		
		OZ_Bars[n].bar[i].value		= OZ_Input.bar[ data[i*2] ].value
		OZ_Bars[n].bar[i].class		= OZ_Input.bar[ data[i*2] ].class
		if(OZ_Bars[n].bar[i].roster > 0)then
			OZ_Bars[n].bar[i].unit = OZ_RaidRoster.member[OZ_Bars[n].bar[i].roster].unit
		elseif(OZ_Bars[n].bar[i].target > 0)then
			OZ_Bars[n].bar[i].unit = OZ_RaidRoster.target[OZ_Bars[n].bar[i].roster].unit
		else
			OZ_Bars[n].bar[i].unit = nil
		end
	end
	OZ_Bars[n].nBars = OZ_Input.nBars
end

function OZ_NoSort(n,scale)
	local i
	for i = 1,OZ_Input.nBars do
		OZ_Bars[n].bar[i].roster	= OZ_Input.bar[i].roster
		OZ_Bars[n].bar[i].target	= OZ_Input.bar[i].target
		OZ_Bars[n].bar[i].current	= OZ_Input.bar[i].current
		OZ_Bars[n].bar[i].max		= OZ_Input.bar[i].max
		OZ_Bars[n].bar[i].value		= OZ_Input.bar[i].value
		OZ_Bars[n].bar[i].class		= OZ_Input.bar[i].class
	end
	OZ_Bars[n].nBars = OZ_Input.nBars
end
------------------------------------------------------------
-- Sort function
--
-- This is a merge-sort, O(n log n), and should be marginally faster than a qsort
-- AND it is a stable sort, so entries with identical values preserve their order
-- Merge sorts are also generally better than heapsorts when dealing with arrays
-- so al in all this is the best sort for this type of mod.
--
-- It takes a single array with pairs of weights and 'source indices' that then let
-- you apply the sort to another array (its faster sorting with just 2 values per entry)
--   ..., weight, srcIndex, ...
-- It then sorts this array so the weights are ASCENDING
-- e.g:
-- input { 1.4, 1, -17.1, 2, 0.3, 3 } -> output { -17.1, 2, 0.3, 3, 1.4, 1 }
-- NOTE: The sort weights do not need to be in any particular range
------------------------------------------------------------

function OZ_internalMergeSort(n,data)
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Merge Sort-----------------")
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Input: "..n.." entries:")
--for key,value in data do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33"..key.." = "..value)
--end

	-- Step 1 - split input array down the middle
	local lsize = math.floor(n*0.25)*2
	local rsize = n - lsize
	local left = {}
	local right = {}
	local i
	for i = 1,lsize do
		left[i]	= data[i]
	end
	for i = 1,rsize do
		right[i] = data[lsize+i]
	end

	-- Step 2 - if there is more than one entry on a 'branch' then recurse to sort it...
	if(lsize > 2) then
		left  = OZ_internalMergeSort( lsize, left)
	end
	if(rsize > 2) then
		right = OZ_internalMergeSort( rsize, right)
	end

	-- Step 3 - 'left' and 'right' are now sorted arrays, so merge them
	-- This is fast because the arrays are already sorted, so just take
	-- the best off each until one of the branches is empty
	local sorted = {}
	local l,r,s = 1,1,1
	
	while ( (l<lsize) and (r<rsize)  ) do
		if( left[l] <= right[r] ) then
			-- append 'left[l]' to 'sorted[s]'
			sorted[s] = left[l]
			sorted[s+1] = left[l+1]
			l = l + 2
		else
			-- append 'right[r] to 'sorted[s]'
			sorted[s] = right[r]
			sorted[s+1] = right[r+1]
			r = r + 2
		end
		s = s + 2
	end

	-- Now append any remaining items
	if( l<lsize ) then
		-- 'left' still has some in...
		for i = l,lsize do
			sorted[s] = left[i]
			s = s + 1
		end
	elseif( r<rsize ) then
		-- 'right' still has some in...
		for i = r,rsize do
			sorted[s] = right[i]
			s = s + 1
		end
	end

--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Output: "..s - 1.." entries:")
--for d=1,s-1,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..sorted[d].."),("..sorted[d+1]..")")
--end
	return sorted
end

---------------------------------------------------------------
-- Function to help debug the sort (also works as example code)
---------------------------------------------------------------
function debugMergeSort()
	OZ_SortMap = {	0.5, 1,
					0.7, 2,
					0.7, 5,
					0.3, 3,
					0.1, 7,
					0.2, 6,
					0.9, 4,
					0.4, 8,
					0.6, 9,
					0.8, 10,
					1.0, 11,
					1.5, 12,
					1.3, 13,
					1.2, 14 };

	data = OZ_internalMergeSort2(28)
end



--------------------------------------------------------------------
-- New SORT implemention - Stack based Merge Sort
--
--	Implements the sort using 'OZ_SortMap' as a FILO stack
--	Removes the need for creating any new arrays, and removes the need
--	for any array data to be copied around!
--	You dont even need to copy the result data as its done in place!
--	this WILL extend the original table by nearly 3x the original length,
-- but no more than that: n(input) + n + n/2 + n/4... etc.

function OZ_internalMergeSort2(n)
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Merge Sort-----------------")
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Input: "..n.." entries:")
--for key,value in OZ_SortMap do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33"..key.." = "..value)
--end

	-- Step 1 - split input array down the middle
	local lsize = math.floor(n*0.25)*2
	local rsize = n - lsize
	local i
	local left = n + 1
	local right = left + lsize
	local top = left + n

	for i = 1,n do
		OZ_SortMap[n+i] = OZ_SortMap[i]
	end



	-- Step 2 - if there is more than one entry on a 'branch' then recurse to sort it...
	if(lsize > 2) then
		left = OZ_internalMergeSort3( lsize, left, top )
	end
	if(rsize > 2) then
		right = OZ_internalMergeSort3( rsize, right, top + lsize )
	end

	-- Step 3

	local l,r,s = left,right,1
	local l_end = right - 1
	local r_end = top - 1
	while ( (l<l_end) and (r<r_end)  ) do
		if( OZ_SortMap[l] <= OZ_SortMap[r] ) then
			-- append 'left[l]' to 'sorted[s]'
			OZ_SortMap[s] = OZ_SortMap[l]
			OZ_SortMap[s+1] = OZ_SortMap[l+1]
			l = l + 2
		else
			-- append 'right[r] to 'sorted[s]'
			OZ_SortMap[s] = OZ_SortMap[r]
			OZ_SortMap[s+1] = OZ_SortMap[r+1]
			r = r + 2
		end
		s = s + 2
	end

	-- Now append any remaining items
	if( l<l_end ) then
		-- 'left' still has some in...
		for i = l,l_end do
			OZ_SortMap[s] = OZ_SortMap[i]
			s = s + 1
		end
	elseif( r<r_end ) then
		-- 'right' still has some in...
		for i = r,r_end do
			OZ_SortMap[s] = OZ_SortMap[i]
			s = s + 1
		end
	end

--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Output:")
--for d=1,n,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..OZ_SortMap[d].."),("..OZ_SortMap[d+1]..")")
--end
end

function OZ_internalMergeSort3(n, data, stackPos)
	-- Step 1 - split input array down the middle
	local lsize = math.floor(n*0.25)*2
	local rsize = n - lsize
	local i
	local left = stackPos
	local right = stackPos + lsize
	local top = stackPos + n
	local r_dest = top + lsize

	for i = 0,n-1 do
		OZ_SortMap[stackPos+i] = OZ_SortMap[data+i]
	end

--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."------")
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Input: "..n.." entries, stack="..stackPos)
--for i = data,data+n-2,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33"..i.." = "..OZ_SortMap[i]..","..OZ_SortMap[i+1])
--end

	-- Step 2 - if there is more than one entry on a 'branch' then recurse to sort it...
	if(lsize > 2) then
		left = OZ_internalMergeSort3( lsize, left, top )
--	else
--		OZ_SortMap[stackPos] = OZ_SortMap[left]
--		OZ_SortMap[stackPos+1] = OZ_SortMap[left+1]
--		left = stackPos
	end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Sorted LEFT, pos = "..left)
--for d=left,left+lsize-2,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..OZ_SortMap[d].."),("..OZ_SortMap[d+1]..")")
--end
	if(rsize > 2) then
		right = OZ_internalMergeSort3( rsize, right, r_dest )
--	else
--		OZ_SortMap[r_dest] = OZ_SortMap[right]
--		OZ_SortMap[r_dest+1] = OZ_SortMap[right+1]
--		right = r_dest
	end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Sorted RIGHT, pos ="..right)
--for d=right,right+rsize-2,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..OZ_SortMap[d].."),("..OZ_SortMap[d+1]..")")
--end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."merge with LEFT, pos = "..left)
--for d=left,left+lsize-2,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..OZ_SortMap[d].."),("..OZ_SortMap[d+1]..")")
--end

	-- Step 3
	local l,r,s = left,right,data
	local l_end = left + lsize - 1
	local r_end = right + rsize - 1
	while ( (l<l_end) and (r<r_end)  ) do
		if( OZ_SortMap[l] <= OZ_SortMap[r] ) then
			-- append 'left[l]' to 'sorted[s]'
			OZ_SortMap[s] = OZ_SortMap[l]
			OZ_SortMap[s+1] = OZ_SortMap[l+1]
			l = l + 2
		else
			-- append 'right[r] to 'sorted[s]'
			OZ_SortMap[s] = OZ_SortMap[r]
			OZ_SortMap[s+1] = OZ_SortMap[r+1]
			r = r + 2
		end
		s = s + 2
	end

	-- Now append any remaining items
	if( l<l_end ) then
		-- 'left' still has some in...
		for i = l,l_end do
			OZ_SortMap[s] = OZ_SortMap[i]
			s = s + 1
		end
	elseif( r<r_end ) then
		-- 'right' still has some in...
		for i = r,r_end do
			OZ_SortMap[s] = OZ_SortMap[i]
			s = s + 1
		end
	end
--DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".."Output ("..data.." to "..(data+n-1)..")")
--for d=data,data+n-2,2 do
--	DEFAULT_CHAT_FRAME:AddMessage("|c0033FF33".." ("..OZ_SortMap[d].."),("..OZ_SortMap[d+1]..")")
--end
	return data
end
